﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace ThinkgeoWmsIneMemoryFeatureLayer
{
    /// <summary>
    ///     Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            Loaded += MainWindow_Loaded;
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            AddSource();
            ApplyMap();
            AddInMemoryFeatureLayer();
          
        }

        private void AddInMemoryFeatureLayer()
        {
            var highlightOverlay = new LayerOverlay();

            var layer = new ShapeFileFeatureLayer(@"ShpFiles\Cardiff_Boundary_region.shp");
            layer.Open();
            IEnumerable<Feature> features = layer.QueryTools.GetAllFeatures(ReturningColumnsType.NoColumns).Take(01);
            Collection<FeatureSourceColumn> collection = layer.QueryTools.GetColumns();
            var inmeory = new InMemoryFeatureLayer(collection.ToList(), features);
            inmeory.ZoomLevelSet = GetHighlightLayerStyle();

            var projection = new Proj4Projection
            {
                InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(27700),
                ExternalProjectionParametersString = Proj4Projection.GetBingMapParametersString()
            };

            projection.Open();

            inmeory.FeatureSource.Projection = projection;


            highlightOverlay.Layers.Add(inmeory);

            WpfMap.Overlays.Add(highlightOverlay);
            WpfMap.Overlays.MoveToTop(highlightOverlay);
            WpfMap.Refresh(highlightOverlay);
        }

        private static ZoomLevelSet GetHighlightLayerStyle()
        {
            var style = new ZoomLevelSet();
            style.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle,
                GeoColor.SimpleColors.Red, 10.0f);
            style.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.SimpleColors.Red, 6.0f, false);
            style.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Red);
            style.ZoomLevel01.DefaultTextStyle = TextStyles.CreateMaskTextStyle("length", "Arial", 10f,
                DrawingFontStyles.Regular,
                GeoColor.StandardColors.Black,
                GeoColor.StandardColors.White);
            style.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            style.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            return style;
        }

        public void ApplyMap()
        {
            var projection = new Proj4Projection
            {
                InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(27700),
                ExternalProjectionParametersString = Proj4Projection.GetBingMapParametersString()
            };

            projection.Open();
            foreach (Overlay itm in WpfMap.Overlays)
            {
                var overlay = (LayerOverlay) itm;
                var featureLayer = (FeatureLayer) overlay.Layers.FirstOrDefault();
                if (featureLayer == null)
                    continue;
                featureLayer.FeatureSource.Projection = projection;
                WpfMap.Refresh(overlay);
            }

            WpfMap.CurrentExtent = projection.ConvertToExternalProjection(WpfMap.CurrentExtent);
            // _projection.Close();


            var bingMapsOverlay = new BingMapsOverlay("Au9yN0FUHyp5zAoh0zWBjbw81-eSeaiH2J_zTFiCWQ0OYlhYX9i17jR9cnBhQFLJ");
            WpfMap.Overlays.Insert(0, "bingMapsOverlay", bingMapsOverlay);

            WpfMap.Refresh();
        }

        private void AddSource()
        {
            WpfMap.MapUnit = GeographyUnit.Meter;

            var layer = new ShapeFileFeatureLayer(@"ShpFiles\Cardiff_Boundary_region.shp");
            layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoColor.SimpleColors.Black),
                new GeoSolidBrush(GeoColor.SimpleColors.Transparent));
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            var staticOverlay = new LayerOverlay
            {
                TransitionEffect = TransitionEffect.None,
                TileType = TileType.SingleTile
            };
            staticOverlay.Layers.Add("layer", layer);
            WpfMap.Overlays.Add(staticOverlay);


            layer.Open();
            RectangleShape _extent = layer.GetBoundingBox();
            layer.Close();
            WpfMap.CurrentExtent = _extent;
            WpfMap.Refresh();
        }
    }
}